@@ -13,8 +13,9 @@ use File::DataClass::Constants qw( CYGWIN EVIL EXCEPTION_CLASS FALSE
                                    LOCK_BLOCKING LOCK_NONBLOCKING
                                    NO_UMASK_STACK NUL PERMS STAT_FIELDS
                                    TILDE TRUE );
-use File::DataClass::Functions qw( first_char is_arrayref is_coderef
-                                   is_hashref is_member thread_id throw );
+use File::DataClass::Functions qw( ensure_class_loaded first_char is_arrayref
+                                   is_coderef is_hashref is_member thread_id
+                                   throw );
 use File::Spec                   ( );
 use File::Spec::Functions      qw( curdir );
 use IO::Dir;
@@ -27,7 +28,9 @@ use Unexpected::Types          qw( ArrayRef Bool CodeRef Int Maybe Object
                                    PositiveInt RegexpRef SimpleStr Str );
 
 use namespace::clean -except => [ 'import', 'meta' ];
-use overload '""' => sub { $_[ 0 ]->pathname }, fallback => 1;
+use overload '""'       => sub { $_[ 0 ]->pathname },
+             'bool'     => sub { $_[ 0 ]->boolean  },
+             'fallback' => TRUE;
 
 our @EXPORT    = qw( io );
 
@@ -60,10 +63,23 @@ my $_coerce_name = sub {
    return $name;
 };
 
-my $_include_path = sub {
+my $_should_include_path = sub {
    return (not defined $_[ 0 ] or (map { $_[ 0 ]->() } ($_[ 1 ]))[ 0 ]);
 };
 
+# Private methods
+my $_build_dir_pattern = sub {
+   my $self = shift; my $pat = NUL;
+
+   my $curdir = curdir; my $updir = File::Spec->updir;
+
+   $curdir and $pat  = "\Q${curdir}\E"; # uncoverable condition left
+   $curdir and $updir and $pat .= '|';  # uncoverable condition left
+   $updir  and $pat .= "\Q${updir}\E";  # uncoverable condition left
+
+   return qr{ \A (?:$pat) \z }mx;
+};
+
 # Public attributes
 has 'autoclose'     => is => 'lazy', isa => Bool,           default => TRUE  ;
 has 'have_lock'     => is => 'rwp',  isa => Bool,           default => FALSE ;
@@ -86,17 +102,16 @@ has '_backwards'    => is => 'rw',   isa => Bool,           default => FALSE ;
 has '_block_size'   => is => 'rw',   isa => PositiveInt,    default => 1024  ;
 has '_chomp'        => is => 'rw',   isa => Bool,           default => FALSE ;
 has '_deep'         => is => 'rw',   isa => Bool,           default => FALSE ;
-has '_dir_pattern'  => is => 'lazy', isa => RegexpRef                        ;
+has '_dir_pattern'  => is => 'lazy', isa => RegexpRef,
+   builder          => $_build_dir_pattern                                   ;
 has '_filter'       => is => 'rw',   isa => Maybe[CodeRef]                   ;
 has '_layers'       => is => 'ro',   isa => ArrayRef[SimpleStr],
-   default          => sub { [] };
+   builder          => sub { [] }                                            ;
 has '_lock'         => is => 'rw',   isa => $IO_LOCK,       default => FALSE ;
-has '_lock_obj'     => is => 'rw',   isa => Maybe[Object],
-   writer           => 'lock_obj'                                            ;
 has '_no_follow'    => is => 'rw',   isa => Bool,           default => FALSE ;
 has '_separator'    => is => 'rw',   isa => Str,            default => $RS   ;
 has '_umask'        => is => 'rw',   isa => ArrayRef[Int],
-   default          => sub { [] };
+   builder          => sub { [] }                                            ;
 
 # Construction
 my $_clone_one_of_us = sub {
@@ -136,16 +151,12 @@ around 'BUILDARGS' => sub {
    my ($orig, $class, @args) = @_; return $_build_attr_from->( @args );
 };
 
-sub _build__dir_pattern {
-   my $self = shift; my $pat = NUL;
+sub BUILD {
+   my $self = shift; my $handle = $self->io_handle;
 
-   my $curdir = curdir; my $updir = File::Spec->updir;
+   not $self->name and $handle and $self->_set_is_open( $handle->opened );
 
-   $curdir and $pat  = "\Q${curdir}\E"; # uncoverable condition left
-   $curdir and $updir and $pat .= '|';  # uncoverable condition left
-   $updir  and $pat .= "\Q${updir}\E";  # uncoverable condition left
-
-   return qr{ \A (?:$pat) \z }mx;
+   return;
 }
 
 # Private methods
@@ -174,7 +185,7 @@ my $_find; $_find = sub {
       my $is_dir = $io->is_dir; defined $is_dir or next;
 
       (($files and not $is_dir) or ($dirs and $is_dir))
-         and $_include_path->( $filter, $io ) and push @all, $io;
+         and $_should_include_path->( $filter, $io ) and push @all, $io;
 
       $is_dir and ($follow or not $io->is_link) and $level != 1
          and push @all, $io->$_find( $files, $dirs, $level ? $level - 1 : 0 );
@@ -182,8 +193,8 @@ my $_find; $_find = sub {
 
    not $self->sort and return @all;
 
-   return $self->reverse ? reverse sort { $a->name cmp $b->name } @all
-                         :         sort { $a->name cmp $b->name } @all;
+   return $self->reverse ? sort { $b->name cmp $a->name } @all
+                         : sort { $a->name cmp $b->name } @all;
 };
 
 my $_get_atomic_path = sub {
@@ -236,7 +247,7 @@ my $_sane_binmode = sub {
 };
 
 my $_throw = sub {
-   my ($self, @args) = @_; eval { $self->unlock }; throw @args;
+   my $self = shift; eval { $self->unlock }; throw @_;
 };
 
 my $_umask_pop = sub {
@@ -263,7 +274,9 @@ my $_umask_push = sub {
 my $_untainted_perms = sub {
    my $self = shift; $self->exists or return; my $perms = 0;
 
-   $self->stat->{mode} =~ m{ \A (\d+) \z }mx and $perms = $1;
+   my $stat = $self->stat // {}; my $mode = $stat->{mode} // NUL;
+
+   $mode =~ m{ \A (\d+) \z }mx and $perms = $1;
 
    return $perms & oct '07777';
 };
@@ -271,7 +284,7 @@ my $_untainted_perms = sub {
 my $_assert_open_backwards = sub {
    my ($self, @args) = @_; $self->is_open and return;
 
-   require File::ReadBackwards;
+   ensure_class_loaded 'File::ReadBackwards';
 
    $self->_set_io_handle( File::ReadBackwards->new( $self->name, @args ) )
       or $self->$_throw( 'File [_1] cannot open backwards: [_2]',
@@ -466,8 +479,7 @@ sub assert_dirpath {
    my $perms = $self->$_mkdir_perms; $self->$_umask_push( oct '07777' );
 
    unless (CORE::mkdir( $dir_name, $perms )) {
-      require File::Path;
-
+      ensure_class_loaded 'File::Path';
       File::Path::make_path( $dir_name, { mode => $perms } );
    }
 
@@ -537,6 +549,10 @@ sub block_size {
    defined $_[ 1 ] and $_[ 0 ]->_block_size( $_[ 1 ] ); return $_[ 0 ];
 }
 
+sub boolean {
+   return ($_[ 0 ]->io_handle || $_[ 0 ]->name) ? TRUE : FALSE;
+}
+
 sub buffer {
    my $self = shift;
 
@@ -671,7 +687,7 @@ sub digest { # Robbed from Path::Tiny
                                       : { algorithm => $args[ 0 ],
                                           %{ $args[ 1 ] } };
 
-   require Digest; my $digest = Digest->new( $args->{algorithm} );
+   ensure_class_loaded 'Digest'; my $digest = Digest->new( $args->{algorithm} );
 
    if ($args->{block_size}) {
       $self->binmode( ':unix' )->lock->block_size( $args->{block_size} );
@@ -680,7 +696,7 @@ sub digest { # Robbed from Path::Tiny
    }
    else { $digest->add( $self->binmode( ':unix' )->lock->all ) }
 
-   return $digest->hexdigest;
+   return $digest;
 }
 
 sub dir {
@@ -710,7 +726,7 @@ sub error_check {
 }
 
 sub exists {
-   return -e $_[ 0 ]->name;
+   return $_[ 0 ]->name ? -e $_[ 0 ]->name : FALSE;
 }
 
 sub file {
@@ -783,6 +799,10 @@ sub head {
    return wantarray ? @res : join NUL, @res;
 }
 
+sub hexdigest {
+   my ($self, @args) = @_; return $self->digest( @args )->hexdigest;
+}
+
 sub is_absolute {
    return File::Spec->file_name_is_absolute( $_[ 0 ]->name );
 }
@@ -850,7 +870,7 @@ sub iterator {
          while (defined (my $path = $dirs[ 0 ]->next)) {
             $deep and $path->is_dir and ($follow or not $path->is_link)
                and unshift @dirs, $path;
-            $_include_path->( $filter, $path ) and return $path;
+            $_should_include_path->( $filter, $path ) and return $path;
          }
 
          shift @dirs;
@@ -885,7 +905,7 @@ sub mkdir {
 sub mkpath {
    my ($self, $perms) = @_; $perms ||= $self->$_mkdir_perms;
 
-   $self->$_umask_push( oct '07777' ); require File::Path;
+   $self->$_umask_push( oct '07777' ); ensure_class_loaded 'File::Path';
 
    File::Path::make_path( $self->name, { mode => $perms } );
 
@@ -1032,17 +1052,17 @@ sub rmdir {
 }
 
 sub rmtree {
-   my ($self, @args) = @_; require File::Path;
+   my ($self, @args) = @_; ensure_class_loaded 'File::Path';
 
    return File::Path::remove_tree( $self->name, @args );
 }
 
 sub seek {
-   my ($self, @args) = @_;
+   my ($self, $posn, $whence) = @_;
 
    $self->is_open or $self->assert_open( $LC_OSNAME eq EVIL ? 'r' : 'r+' );
 
-   $self->io_handle->seek( @args ); $self->error_check;
+   CORE::seek $self->io_handle, $posn, $whence; $self->error_check;
    return $self;
 }
 
@@ -1066,14 +1086,10 @@ sub set_lock {
    my $self = shift; $self->_lock or return;
 
    my $async = $self->_lock == LOCK_NONBLOCKING ? TRUE : FALSE;
-
-   $self->_lock_obj
-      and return $self->_lock_obj->set( k => $self->name, async => $async );
-
-   my $mode = $self->mode eq 'r' ? LOCK_SH : LOCK_EX;
+   my $mode  = $self->mode eq 'r' ? LOCK_SH : LOCK_EX;
 
    $async and $mode |= LOCK_NB;
-   $self->_set_have_lock( flock $self->io_handle, $mode ? TRUE : FALSE );
+   $self->_set_have_lock( (flock $self->io_handle, $mode) ? TRUE : FALSE );
    return $self;
 }
 
@@ -1096,11 +1112,12 @@ sub splitpath {
 }
 
 sub stat {
-   my $self = shift; $self->name or return {};
+   my $self = shift; $self->exists or $self->is_open or return;
 
    my %stat_hash = ( id => $self->filename );
 
-   @stat_hash{ STAT_FIELDS() } = stat $self->name;
+   @stat_hash{ STAT_FIELDS() }
+      = stat( $self->exists ? $self->name : $self->io_handle );
 
    return \%stat_hash;
 }
@@ -1129,16 +1146,24 @@ sub tail {
    return wantarray ? @res : join NUL, @res;
 }
 
+sub tell {
+   my $self = shift;
+
+   $self->is_open or $self->assert_open( $LC_OSNAME eq EVIL ? 'r' : 'r+' );
+
+   return CORE::tell $self->io_handle;
+}
+
 sub tempfile {
-   my ($self, $tmplt) = @_; my ($tempdir, $tmpfh); require File::Temp;
+   my ($self, $tmplt) = @_; my $tempdir;
+
+   ensure_class_loaded 'File::Temp'; $tmplt ||= '%6.6dXXXX';
 
    ($tempdir = $self->name and -d $tempdir) or $tempdir = File::Spec->tmpdir;
 
-   $tmplt ||= '%6.6dXXXX';
-   $tmpfh   = File::Temp->new
+   my $tmpfh = File::Temp->new
       ( DIR => $tempdir, TEMPLATE => (sprintf $tmplt, $PID) );
-
-   my $t = $self->$_constructor( $tmpfh->filename )->file;
+   my $t     = $self->$_constructor( $tmpfh->filename )->file;
 
    $t->_set_io_handle( $tmpfh );
    $t->_set_is_open( TRUE );
@@ -1162,9 +1187,7 @@ sub unlink {
 sub unlock {
    my $self = shift; $self->_lock or return; my $handle = $self->io_handle;
 
-   if ($self->_lock_obj) { $self->_lock_obj->reset( k => $self->name ) }
-   else { $handle and $handle->opened and flock $handle, LOCK_UN }
-
+   $handle and $handle->opened and flock $handle, LOCK_UN;
    $self->_set_have_lock( FALSE );
    return $self;
 }
@@ -1195,7 +1218,7 @@ __END__
 
 =head1 Name
 
-File::DataClass::IO - Better IO syntax
+File::DataClass::IO - A powerful and concise API to do as much IO as possible
 
 =head1 Synopsis
 
@@ -1313,6 +1336,10 @@ An object which is a L<File::DataClass::IO>
 
 =back
 
+=head2 BUILD
+
+Open the file handle if it is closed and was supplied without a file name
+
 =head2 abs2rel
 
    $path = io( 'path_to_file' )->abs2rel( 'optional_base_path' );
@@ -1453,6 +1480,13 @@ Sets binmode to the given layer
 
 Defaults to 1024. The default block size used by the L</read> method
 
+=head2 boolean
+
+   $bool = io( 'path_to_file' )->boolean;
+
+Returns true if the pathname has been set or the file handle is open, returns
+false otherwise. The boolean overload calls this
+
 =head2 buffer
 
 The internal buffer used by L</read> and L</write>
@@ -1557,9 +1591,9 @@ object is still open it calls the L</close> method
 
 =head2 digest
 
-   $hex_digest = io( 'path_to_file' )->digest( $algorithm, $options );
+   $digest_object = io( 'path_to_file' )->digest( $algorithm, $options );
 
-Returns a hexadecimal string which is calculated from the contents of the
+Returns a L<Digest> object which is calculated from the contents of the
 specified file. The arguments are optional. The algorithm defaults to
 C<SHA-256>. The C<$options> hash reference takes the C<block_size> parameter
 which causes the file to read through the buffer C<block_size> bytes at a
@@ -1649,6 +1683,12 @@ Returns the first I<n> lines from the file where the number of lines
 returned defaults to 10. Returns the lines joined with null in a
 scalar context
 
+=head2 hexdigest
+
+   $hex_digest = io( 'path_to_file' )->hexdigest( $algorithm, $options );
+
+Returns a hexadecimal string which is calculated from the L</digest> object
+
 =head2 _init
 
 Sets default values for some attributes, takes two optional arguments;
@@ -1930,7 +1970,8 @@ Proxy for L<File::Spec/splitpath>
 
    $stat_hash_ref = io( 'path_to_file' )->stat;
 
-Returns a hash of the values returned by a L</stat> call on the pathname
+Returns a hash of the values returned by a L</stat> call on the pathname.
+Returns undefined if the file does not exist or the file handle is not open
 
 =head2 substitute
 
@@ -1947,6 +1988,12 @@ Returns the last I<n> lines from the file where the number of lines
 returned defaults to 10. Returns the lines joined with null in a
 scalar context
 
+=head2 tell
+
+   $byte_offset = io( 'path_to_file' )->tell;
+
+Returns the byte offset into the file
+
 =head2 tempfile
 
    $io = io( 'path_to_temp_directory' )->tempfile( $template );
@@ -2064,7 +2111,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2014 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
