@@ -131,11 +131,12 @@ sub meta_unpack { # Modified in a subclass
 sub read_file {
    my ($self, $path, $for_update) = @_;
 
-   $self->_lock->set( k => $path ); my ($data, $meta, $path_mtime);
+   $self->_lock->set( k => $path ); my ($data, $path_mtime);
 
    try {
-      ($data, $meta)  = $self->_cache->get( $path );
-      $path_mtime     = $path->stat->{mtime};
+      my $stat = $path->stat; defined $stat and $path_mtime = $stat->{mtime};
+
+      my $meta; ($data, $meta) = $self->_cache->get( $path );
 
       my $cache_mtime = $self->meta_unpack( $meta );
 
@@ -158,7 +159,8 @@ sub read_file {
 }
 
 sub read_from_file {
-   throw 'Method [_1] not overridden in subclass', [ 'read_from_file' ];
+   throw 'Method [_1] not overridden in subclass [_2]',
+         [ 'read_from_file', blessed $_[ 0 ] ];
 }
 
 sub select {
@@ -237,16 +239,17 @@ sub write_file {
 }
 
 sub write_to_file {
-   throw 'Method [_1] not overridden in subclass', [ 'write_to_file' ];
+   throw 'Method [_1] not overridden in subclass [_2]',
+         [ 'write_to_file', blessed $_[ 0 ] ];
 }
 
 # Backcompat
 sub _read_file {
-   throw 'Should never call _read_file';
+   throw 'Class [_1] should never call _read_file', [ blessed $_[ 0 ] ];
 }
 
 sub _write_file {
-   throw 'Should never call _write_file';
+   throw 'Class [_1] should never call _write_file', [ blessed $_[ 0 ] ];
 }
 
 1;
@@ -409,7 +412,7 @@ Peter Flanigan, C<< <pjfl@cpan.org> >>
 
 =head1 License and Copyright
 
-Copyright (c) 2014 Peter Flanigan. All rights reserved
+Copyright (c) 2015 Peter Flanigan. All rights reserved
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. See L<perlartistic>
